module CoreModels
  module Models
    module Concerns
      module UserConcerns
        def has_group?(name)
          self.groups.each do |g|
            if g.group_name == name
              return true
            end
          end

          return false
        end

        def has_role?(name)
          self.roles.each do |r|
            if r.role_name == name
              return true
            end
          end

          return false
        end

        def has_role_for_group?(name, group_name)
          if self.groups.where(group_name: group_name).any?
            if self.groups.find(group_name.parameterize).roles.where(role_name: name).any?
              self.has_role?(name)
            end
          end
        end

        def add_group_membership(group, role)
          self.group_memberships.create!(group_id: group.id, role_id: role.id)
        end

        def get_role_from_group_membership(group_id)
          self.group_memberships.find_by!(group_id: group_id).role
        end

        def remove_group_membership(group_name)
          group = self.groups.klass.find_by!(group_name: group_name)
          self.group_memberships.where(group_id: group.id).delete_all
        end

        def add_group=(group_name)
          self.groups << self.groups.klass.find_by(group_name: group_name)
        end

        def role
          self.roles.first.try(:role_name)
        end

        def group
          self.groups.first.try(:group_name)
        end

        def group_names
          associated_groups = Array.new
          self.groups.each do |group|
            associated_groups.push(group.group_name)
          end

          associated_groups
        end

        def role_names
          associated_roles = Array.new
          self.roles.each do |role|
            associated_roles.push(role.role_name)
          end

          associated_roles
        end

        def uniq_permissions
          permission_array = Array.new

          self.roles.each do |role|
            role.permissions.each do |permission|
              permission_array.push(permission.permission_name)
            end
          end

          permission_array.uniq
        end

        def permissions
          permission_array = Array.new

          self.roles.each do |role|
            role.permissions.each do |permission|
              permission_array.push(permission.permission_name)
            end
          end

          permission_array
        end
      end
    end
  end
end
