require_relative 'concerns/role_concerns'
require 'active_support/concern'

module CoreModels
  module Models
    module Role
      extend ActiveSupport::Concern
      include CoreModels::Models::Concerns::RoleConcerns

      included do
        extend FriendlyId

        friendly_id :role_name, use: [:slugged, :finders, :history]

        has_many :group_memberships
        has_many :users, :through => :group_memberships

        has_many :groups_roles
        has_many :groups, :through => :groups_roles

        has_many :roles_permissions
        has_many :permissions, :through => :roles_permissions

        validates :role_name, presence: true
        validates_uniqueness_of :role_name
      end

      def should_generate_new_friendly_id?
        role_name_changed?
      end
    end
  end
end
