require_relative 'concerns/user_concerns'
require 'active_support/concern'
require 'bcrypt'

module CoreModels
  module Models
    module User
      extend ActiveSupport::Concern
      include CoreModels::Models::Concerns::UserConcerns

      included do
        extend FriendlyId
        friendly_id :first_name, use: [:slugged, :finders, :history]

        before_save :encrypt_password

        has_many :group_memberships, :dependent => :delete_all
        has_many :groups, :through => :group_memberships, :dependent => :delete_all
        has_many :roles, :through => :group_memberships, :dependent => :delete_all

        validates :first_name, presence: true
        validates :user_name, uniqueness: true, presence: true, length: {minimum: 5}
        validates :email, presence: true, confirmation: true, uniqueness: true
        validates_format_of :email, :with => /\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\z/i
        validates :password, presence: true, confirmation: true, length: { minimum: 10 }

        before_create{ generate_token(:auth_token) }
      end

      class_methods do
        def authenticate_user(user_name, password)
          user = self.find_by_user_name(user_name)
          if(user && (user.password == BCrypt::Engine.hash_secret(password, user.salt)))
            user
          else
            nil
          end
        end
      end

      def encrypt_password
        if password.present?
          self.salt = BCrypt::Engine.generate_salt
          self.password = BCrypt::Engine.hash_secret(password, salt)
        end
      end

      def send_password_reset
        generate_token(:password_reset_token)
        self.password_reset_timestamp = Time.zone.now
        save!
        UserMailer.password_reset(self).deliver
      end

      protected
      def generate_token(column)
        begin
          self[column] = SecureRandom.urlsafe_base64
        end while self.class.exists?(column => self[column])
      end
    end
  end
end
